﻿//////////////////////////////////////////////////
//                  Spell.cs                    //
//         Part of RebornCo by kaihaider        //
//////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using System.Windows.Media;
using Buddy.Coroutines;
using Clio.Utilities;
using ff14bot;
using ff14bot.Enums;
using ff14bot.Helpers;
using ff14bot.Managers;
using ff14bot.Navigation;
using ff14bot.Objects;
using RebornCo.Settings;

namespace RebornCo.Helpers
{
    public static class Spell
    {
        #region PullApply

        public static async Task<bool> PullApply(string name)
        {
            return await PullApply(name, Core.Me.CurrentTarget, true);
        }
        public static async Task<bool> PullApply(string name, bool cond)
        {
            return await PullApply(name, Core.Me.CurrentTarget, cond);
        }
        public static async Task<bool> PullApply(string name, GameObject o)
        {
            return await PullApply(name, o, true);
        }
        public static async Task<bool> PullApply(string name, GameObject o, bool cond)
        {
            if (!cond) return false;
            o = o ?? Core.Player.CurrentTarget;
            if (o == null) return false;
            SpellRangeCheck ck = Actionmanager.InSpellInRangeLOS(name, o);
            if (!(Core.Me.IsCasting || !RebornCoRoutine.WindowSettings.Movement)
                && (ck == SpellRangeCheck.ErrorNotInLineOfSight
                    || ck == SpellRangeCheck.ErrorNotInRange && !(MovementManager.IsMoving && Movement.IsFacingMovement(Core.Me, o))
                //|| RebornCoRoutine.WindowSettings.WaitIfMoveing && Core.Me.Distance(Core.Me.CurrentTarget.Location) < Core.Me.Distance(Movement.OldTargetLocation) && Movement.IsFacingMovement(o, Core.Me) 
                ))
            {
                await Coroutine.ExternalTask(Task.Run(() => Movement.MoveTo(o)));
                return true;
            }
           // if (ck == SpellRangeCheck.ErrorNotInFront) return await Movement.CheckFace(o);
            return await Spell.Apply(name, o);
        }

        #endregion

        #region PullCast

        public static async Task<bool> PullCast(string name)
        {
            return await PullCast(name, Core.Me.CurrentTarget, true);
        }
        public static async Task<bool> PullCast(string name, bool cond)
        {
            return await PullCast(name, Core.Me.CurrentTarget, cond);
        }
        public static async Task<bool> PullCast(string name, GameObject o)
        {
            return await PullCast(name, o, true);
        }
        public static async Task<bool> PullCast(string name, GameObject o, bool cond)
        {
            if (!RebornCoRoutine.WindowSettings.Movement || Core.Me.IsCasting || !cond) return false;
            o = o ?? Core.Player.CurrentTarget;
            if (o == null && Core.Target != null && !Movement.IsInSafeRange(Core.Target))
            {
                Logging.Write(Colors.SkyBlue, "[RebornCo] Searching for Core.Target");
                await Coroutine.ExternalTask(Task.Run(() => Movement.MoveTo(Core.Target)));
                return true; 
            }//possible add target
            else if (o == null) { return false; }
            SpellRangeCheck ck = Actionmanager.InSpellInRangeLOS(name, o);
            if ((ck == SpellRangeCheck.ErrorNotInLineOfSight
                    || ck == SpellRangeCheck.ErrorNotInRange && !(MovementManager.IsMoving && Movement.IsFacingMovement(Core.Me, o))
                //|| RebornCoRoutine.WindowSettings.WaitIfMoveing && Core.Me.Distance(Core.Me.CurrentTarget.Location) < Core.Me.Distance(Movement.OldTargetLocation) && Movement.IsFacingMovement(o, Core.Me) 
                ))
            {
                await Coroutine.ExternalTask(Task.Run(() => Movement.MoveTo(o)));
                return true;
            }
            //if (ck == SpellRangeCheck.ErrorNotInFront) return await Movement.CheckFace(o);
            return await Spell.Cast(name, o);
        }

        #endregion
        public static bool CurrentJob(ClassJobType j)
        {
            if (j == Core.Me.CurrentJob) return true;
            if (j == ClassJobType.Lancer && Core.Me.CurrentJob == ClassJobType.Dragoon) return true;
            if (j == ClassJobType.Rogue && Core.Me.CurrentJob == ClassJobType.Ninja) return true;
            if (j == ClassJobType.Pugilist && Core.Me.CurrentJob == ClassJobType.Monk) return true;
            return false;
        }
        #region combo
        public static bool ComboCountCheck;
        public static Combo LastCombo;
        public static TimeSpan ComboCountDown;
        public static Dictionary<uint,Combo> ComboSpells = new Dictionary<uint,Combo> ()
        {
            {75, Combo.TrueThrust},
            {78, Combo.VorpalThrust},
            {79, Combo.HeavyThrust},
            {81, Combo.ImpulseDrive},
            {87, Combo.Disembowel},
            {2240, Combo.SpinningEdge},
            {2242, Combo.GustSlash}

        };


        public static void SyncCombos()
        {
            if (ComboCountCheck && ComboCountDown < DateTime.UtcNow.TimeOfDay)
            {
                if (ComboSpells.ContainsKey(Actionmanager.LastSpellId))
                    LastCombo = ComboSpells[Actionmanager.LastSpellId];
                else
                {
                    ComboCountCheck = false;
                    if (LastCombo != Combo.Empty)
                        Logging.Write(Colors.OrangeRed, "[RebornCo] Clearing " + LastCombo + " because " + Actionmanager.LastSpell);
                    LastCombo = Combo.Empty;

                }
            }
        }
        public static bool MidCombo()
        {
            if((Core.Me.CurrentJob==ClassJobType.Lancer || Core.Me.CurrentJob==ClassJobType.Dragoon)
                &&( LastCombo == Combo.TrueThrust || LastCombo == Combo.VorpalThrust && Core.Me.ClassLevel > 25
                    || LastCombo == Combo.ImpulseDrive && Core.Me.ClassLevel > 37 || LastCombo == Combo.Disembowel && Core.Me.ClassLevel ==50)) return true;

            if ((Core.Me.CurrentJob == ClassJobType.Rogue || Core.Me.CurrentJob == ClassJobType.Ninja)
                && (LastCombo == Combo.SpinningEdge && Core.Me.ClassLevel > 3 || LastCombo == Combo.GustSlash && Core.Me.ClassLevel > 25)) return true;

            return false;
        }
#endregion
        #region gcd
        public static bool GetGcd()
        {
            switch (Core.Me.CurrentJob)
            {
                case ClassJobType.Ninja:
                case ClassJobType.Rogue:
                    return RogueGcd();
                case ClassJobType.Dragoon:
                case ClassJobType.Lancer:
                    return LancerGcd();
                case ClassJobType.Monk:
                case ClassJobType.Pugilist:
                    return PugGcd();
                default:
                    return false;
            }
        }

        public static bool RogueGcd()
        {
            SpellData data = DataManager.GetSpellData("Spinning Edge");
            return data.Cooldown.TotalMilliseconds > RebornCoRoutine.WindowSettings.SpellQueueWindow;
        }
        public static bool PugGcd()
        {
            SpellData data = DataManager.GetSpellData("Bootshine");
            return data.Cooldown.TotalMilliseconds > RebornCoRoutine.WindowSettings.SpellQueueWindow;
        }
        public static bool LancerGcd()
        {
            SpellData data = DataManager.GetSpellData("True Thrust");
            return data.Cooldown.TotalMilliseconds > RebornCoRoutine.WindowSettings.SpellQueueWindow;
        }
        #endregion
        public static bool CanCast(SpellData data, GameObject o)
        {
            //if (data.LevelAcquired > Core.Player.Levels[ClassJobType.Adventurer]) return false;
            SpellRangeCheck ck = Actionmanager.InSpellInRangeLOS(data, o);
            if (ck == SpellRangeCheck.ErrorNotInRange || ck == SpellRangeCheck.ErrorNotInLineOfSight) return false;
            //may want to remove later
            if (Core.Me.CurrentTarget != null && Core.Me.Distance(Core.Me.CurrentTarget.Location)< Core.Me.Distance(Movement.OldTargetLocation)
                && Movement.IsInSafeRange(o) &&!Movement.TooClose(o)) Movement.StopMove();
            if (ck == SpellRangeCheck.ErrorNotInFront)
            {
                Logging.Write(Colors.Red, "[RebornCo] Attempted to cast while not facing!");
                Movement.Face(o);
            }
            if (data.Cooldown.TotalMilliseconds > RebornCoRoutine.WindowSettings.SpellQueueWindow) return false;
            return true;
        }
        #region Cast - by name
        public static async Task<bool> CastLock(string name)
        {
            return await Cast(name, Core.Me.CurrentTarget, true, false,true);
        }

        public static async Task<bool> CastLock(string name, GameObject o)
        {
            return await Cast(name, o, true, false,true);
        }

        public static async Task<bool> CastLock(string name, bool cond)
        {
            return await Cast(name, Core.Me.CurrentTarget, cond, false, true);
        }
        public static async Task<bool> CastLock(string name, GameObject o, bool cond)
        {
            return await Cast(name, o, cond, false,true);
        }
        public static async Task<bool> Cast(string name)
        {
            return await Cast(name, Core.Me.CurrentTarget, true);
        }

        public static async Task<bool> Cast(string name, GameObject o)
        {
            return await Cast(name, o, true);
        }

        public static async Task<bool> Cast(string name, bool cond)
        {
            return await Cast(name, Core.Me.CurrentTarget, cond);
        }
        public static async Task<bool> Cast(string name, GameObject o, bool cond, bool ignoreCanCast = false, bool lockDoubleCast = false, bool ignoreGcd = false)
        {
            var killTick = false;
            SpellData data=null;
            if (await Coroutine.ExternalTask(Task.Run(() =>
            {
                if (Core.Player == null) return false;
                if (Actionmanager.CurrentActions.TryGetValue(name, out data))
                {
                    if (lockDoubleCast && Extensions.DoubleCastPreventionDict.Contains(o, name)) return false;
                    if (!CurrentJob(data.Job) &&
                        (!cond || !ignoreCanCast && !Actionmanager.CanCast(data, o ?? Core.Player.CurrentTarget)))
                        return false;
                    if (!cond || !ignoreCanCast && !CanCast(data, o ?? Core.Player.CurrentTarget)) return false;
                    var castTime = data.AdjustedCastTime.TotalSeconds > 0;
                    if (castTime)
                    {
                        Movement.StopMove();
                        //return BotManager.Current.IsAutonomous;
                    }
                    if (Actionmanager.DoAction(name, (o ?? Core.Player.CurrentTarget)))
                    {
                        killTick = true;
                        return true;
                    }
                    killTick = true;
                    return false;
                }
                return false;
            })) && data != null)
            {
                if (!ignoreGcd&&!GetGcd())
                {
                    if (!await Coroutine.Wait(RebornCoRoutine.WindowSettings.GcdSyncWait, GetGcd)) Logging.Write(Colors.LawnGreen, "[RebornCo] gcd sync timed out"); ;
                }
                await Coroutine.ExternalTask(Task.Run(() =>
                {
                    Logging.Write(Colors.Orchid, "[RebornCo] Casting " + name);
                    if (!ignoreGcd && !GetGcd())
                    {
                        if ((o as BattleCharacter) != null && (o as BattleCharacter).IsAlive)
                            Logging.Write(Colors.LawnGreen, "[RebornCo] No GCD after cast, something wrong?");
                        return;
                    }
                    if (lockDoubleCast)
                        Extensions.UpdateDoubleCastDictShort(name, o ?? Core.Player.CurrentTarget);
                    if (ComboSpells.ContainsKey(data.Id))
                    {
                        ComboCountDown = DateTime.UtcNow.TimeOfDay + TimeSpan.FromSeconds(2.5);
                        ComboCountCheck = true;
                        LastCombo = ComboSpells[data.Id];
                    }
                }));
            }
            return killTick;
        }

        public static async Task<bool> Cast(string name, Vector3 loc, bool cond, bool ignoreCanCast = false)
        {
            return await Coroutine.ExternalTask(Task.Run(() =>
            {
                SpellData data;
                if (Actionmanager.CurrentActions.TryGetValue(name, out data))
                {
                    if (!cond) return false;
                    if (!ignoreCanCast && !Actionmanager.CanCastLocation(name, loc))
                        return false;
                    var castTime = data.AdjustedCastTime.TotalSeconds > 0;
                    if (castTime && MovementManager.IsMoving)
                    {
                        return BotManager.Current.IsAutonomous;
                    }
                    Movement.StopMove();

                    // double t = data.Cooldown.TotalMilliseconds;
                    if (Actionmanager.DoActionLocation(name, loc))
                    {
                        Logging.Write(Colors.OrangeRed, "[RebornCo] Casting {0} at {1}", name, loc);
                        // if (t > 0) Logging.Write(Colors.Orchid, "[RebornCo] " + name + " cooldown: " + t);
                        return true;
                    }
                }
                return false;
            }));
        }

        #endregion

        #region apply
        public static async Task<bool> Apply(string name)
        {
            return await Apply(name, true, null, 0, false);
        }
        public static async Task<bool> Apply(string name, bool cond)
        {
            return await Apply(name, cond, null, 0, false);
        }
        public static async Task<bool> Apply(string name, bool cond, GameObject o)
        {
            return await Apply(name, cond, o, 0, false);
        }
        public static async Task<bool> Apply(string name,GameObject o)
        {
            return await Apply(name, true, o, 0, false);
        }
        public static async Task<bool> Apply(string name, bool cond, GameObject o, int msLeft = 0, bool ignoreCanCast = false)
        {

            return await Coroutine.ExternalTask(Task.Run(() =>
            {
                SpellData data;
                if (Actionmanager.CurrentActions.TryGetValue(name, out data))
                {
                    SpellRangeCheck ck = Actionmanager.InSpellInRangeLOS(data, o);
                    o = o ?? Core.Player.CurrentTarget;
                    if (!cond || o==null|| !ignoreCanCast && !CanCast(data, o))
                        return false;
                    
                    if (Extensions.DoubleCastPreventionDict.Contains(o, name))
                        return false;
                    if (!o.HasAura(name, true, msLeft))
                    {
                        var castTime = data.AdjustedCastTime.TotalSeconds > 0;
                        if (castTime && MovementManager.IsMoving)
                        {
                            return BotManager.Current.IsAutonomous;
                        }
                    }
                    if (MovementManager.IsMoving)
                        Navigator.PlayerMover.MoveStop();

                    Logging.Write(Colors.OrangeRed, "[RebornCo] Applying " + name);
                    if (Actionmanager.DoAction(name, o))
                    {
                        Extensions.UpdateDoubleCastDict(name, o);
                        return false;
                    }
                    return true;
                }
                return false;
            }));
        }
        #endregion
    }
}
