﻿//////////////////////////////////////////////////
//                 RebornCo.cs                  //
//         Part of RebornCo by kaihaider        //
//////////////////////////////////////////////////

using Clio.Common;
using RebornCo.Settings;
using System;
using System.Windows.Forms;
using System.Windows.Media;
using ff14bot.Enums;
using ff14bot.NeoProfiles;
using RebornCo.Helpers;
using ff14bot;
using ff14bot.Behavior;
using ff14bot.Helpers;
using ff14bot.Managers;
using ff14bot.Objects;
using ff14bot.AClasses;
using Extensions = RebornCo.Helpers.Extensions;

[assembly: System.Reflection.AssemblyVersion("0.1.4")]
namespace RebornCo
{
    public abstract partial class RebornCoRoutine : CombatRoutine
    {
        
        public override float PullRange
        {
            get { return 2.5f; }
        }
        public override string Name { get { return "RebornCo"; } }
        private Form _configForm;

        public static ICore WindowSettings
        {
            get
            {
                switch (Core.Me.CurrentJob)
                {
                    case ClassJobType.Ninja:
                    case ClassJobType.Rogue:
                        return RogueSetting.Instance;
                        break;
                    case ClassJobType.Dragoon:
                        return DragoonSetting.Instance;
                        break;
                    case ClassJobType.Monk:
                    case ClassJobType.Pugilist:
                        return PugSetting.Instance;
                        break;
                    default:
                        return GeneralSetting.Instance;
                        break;
                }
            }
        }

        public override bool WantButton { get { return true; } }
        public static ClassJobType OldJob { get; set; } 
        
        public sealed override ClassJobType[] Class
        {
            get
            {
                return new ClassJobType[] 
                { 
                  //  ClassJobType.Arcanist, //ClassJobType.Summoner, ClassJobType.Scholar,
                    //ClassJobType.Archer, ClassJobType.Bard, 
                    //ClassJobType.Conjurer, ClassJobType.WhiteMage,
                   // ClassJobType.Gladiator, ClassJobType.Paladin,
                    ClassJobType.Lancer, ClassJobType.Dragoon,
                   // ClassJobType.Marauder, ClassJobType.Warrior,
                    ClassJobType.Pugilist, ClassJobType.Monk,
                   // ClassJobType.Thaumaturge, ClassJobType.BlackMage
                   ClassJobType.Rogue, ClassJobType.Ninja
                };
            }
        }
        public override void OnButtonPress()
        {
            if (_configForm == null || _configForm.IsDisposed || _configForm.Disposing)
                _configForm = new SettingsForm();

            _configForm.ShowDialog();
        }

        #region CombatRoutine implementation

        public virtual void OnPulse()
        {
            Movement.CheckStuckRunning();
            Spell.SyncCombos();
            if (Core.Me.CurrentTarget != null) Movement.OldTargetLocation = Core.Me.CurrentTarget.Location;
            if ((Core.Me.CurrentTarget as BattleCharacter) != null && (Core.Me.CurrentTarget as BattleCharacter).IsCasting && (Core.Me.CurrentTarget as BattleCharacter).SpellCastInfo !=null)
                Logging.Write(Colors.OrangeRed, "[RebornCo] "+(Core.Me.CurrentTarget as BattleCharacter).SpellCastInfo.Name +" Spell ID " + (Core.Me.CurrentTarget as BattleCharacter).SpellCastInfo.ActionId);
            
            if (!OldJob.Equals(Core.Me.CurrentJob))
            {
                OldJob = Core.Me.CurrentJob;
                Logging.Write(Colors.OrangeRed, "(RebornCo) Your job has been updated to " + OldJob + ". Rebuilding behaviors...");
                RoutineManager.Reload();
            }
        }

        public static bool WantHealing = false;

        public override sealed void Pulse()
        {
            Extensions.DoubleCastPreventionDict.RemoveAll(t => DateTime.UtcNow.TimeOfDay > t);
            if (WantHealing)
                HealTargeting.Instance.Pulse();

            OnPulse();
        }
        public static bool Resting = false;        

        #region Hidden Overrides

        public virtual void OnInitialize()
        {

            ff14bot.Navigation.Navigator.PlayerMover = new ff14bot.Navigation.SlideMover();
        }

        internal static RebornCoRoutine Instance;

        public override sealed void Initialize()
        {
            OldJob = Core.Me.CurrentJob;
            Logging.Write("Starting " + Name);
            Instance = this;

            GameEvents.OnMapChanged += (s, e) => UpdateContext();

            TreeHooks.Instance.OnHooksCleared += (s, e) =>
            {
                Logging.Write(Colors.White, "Hooks cleared, re-creating behaviors");
                RebuildBehaviors(silent: true);
            };

            //UpdateContext();

            // NOTE: Hook these events AFTER the context update.
            OnGameContextChanged += (orig, ne) =>
            {
                Logging.Write(Colors.White, "Context changed, re-creating behaviors");
                RebuildBehaviors();
            };

            RoutineManager.Reloaded += (s, e) =>
            {
                Logging.Write(Colors.White, "Routines were reloaded, re-creating behaviors");
                RebuildBehaviors(silent: true);
            };

            if (!RebuildBehaviors())
            {
                return;
            }

            Logging.Write(LogLevel.Verbose, Colors.White, "Verified behaviors can be created!");
            Logging.Write("Initialization complete!");
            OnInitialize();
        }

        public override void ShutDown()
        {
        }
        #endregion
        #endregion
    }
}