﻿//////////////////////////////////////////////////
//                 Lancer.cs                    //
//         Part of RebornCo by kaihaider        //
//////////////////////////////////////////////////
using System.Threading.Tasks;
using ff14bot;
using ff14bot.Enums;
using ff14bot.Managers;
using RebornCo.Helpers;
using TreeSharp;

namespace RebornCo.Rotations
{
    public partial class Routines
    {

        [Behavior(BehaviorType.PreCombatBuffs, ClassJobType.Lancer)]
        public Composite LancerCreateBasicPreCombatBuffs()
        {
            return new ActionRunCoroutine(ctx => LancerBuildPreCombatBuffs());
        }
        public static async Task<bool> LancerBuildPreCombatBuffs()
        {

            if (Core.Me.IsMounted) return false;
            return await Movement.CombatCompanion();
        }

        [Behavior(BehaviorType.Rest, ClassJobType.Lancer)]
        public Composite LancerCreateBasicRest()
        {
            return new ActionRunCoroutine(ctx => DefaultRestBehavior(Core.Player.CurrentTPPercent));
        }


        [Behavior(BehaviorType.CombatBuffs,  ClassJobType.Lancer)]
        public Composite LancerCreateBasicCombatBuffs()
        {
            return new ActionRunCoroutine(ctx => LancerBuildCombatBuffs()); 
        }

        public static async Task<bool> LancerBuildCombatBuffs()
        {
            await Spell.Cast("Keen Flurry", Core.Me, Core.Me.CurrentHealthPercent < 80, false, false, true);
            await Spell.Cast("Foresight", Core.Me, Core.Me.CurrentHealthPercent < 60, false, false, true);
            await Spell.Cast("Convalescence", Core.Me, Core.Me.CurrentHealthPercent < 70, false, false, true);
            await Spell.Cast("Cure", Core.Me, Core.Me.CurrentHealthPercent < 4, false, false, true);
            await Spell.Cast("Physick", Core.Me, Core.Me.CurrentHealthPercent < 40, false, false, true);
            await Spell.Cast("Invigorate", Core.Me, Core.Me.CurrentTPPercent < 50, false, false, true);
            await Spell.Cast("Second Wind", Core.Me, Core.Me.CurrentHealthPercent < 40, false, false, true);
            await Spell.Cast("Awareness", Core.Me, Core.Me.CurrentHealthPercent < 30, false, false, true);
            await Spell.Cast("Bloodbath", Core.Me, Core.Me.CurrentHealthPercent < 70, false, false, true);
            await Spell.Cast("Internal Release", Core.Me, true, false, false, true);
            await Spell.Cast("Raging Strikes", Core.Me, true, false, false, true);
            await Spell.Cast("Blood For Blood", Core.Me, true, false, false, true);
            await Spell.Cast("Featherfoot", Core.Me, Core.Me.Distance2D() < 5, false, false, true);
            return false;
        }

        [Behavior(BehaviorType.Pull, ClassJobType.Lancer)]
        public Composite LancerCreateBasicPull()
        {
            return new ActionRunCoroutine(ctx => LancerBuildPullBehavior());
        }
        public static async Task<bool> LancerBuildPullBehavior()
        {
            await Target.EnsureValidTarget();
            if (await Spell.PullCast("Vorpal Thrust", Core.Me.CurrentTarget, Spell.LastCombo == Combo.TrueThrust)) return true;
            if (await Spell.PullCast("Full Thrust", Core.Me.CurrentTarget, Spell.LastCombo == Combo.VorpalThrust)) return true;
            if (await Spell.PullCast("Disembowel", Core.Me.CurrentTarget, Spell.LastCombo == Combo.ImpulseDrive)) return true;
            if (await Spell.PullCast("Chaos Thrust", Core.Me.CurrentTarget, Spell.LastCombo == Combo.Disembowel)) return true;
            if (await Spell.PullCast("Impulse Drive", Core.Me.CurrentTarget, Core.Me.CurrentTarget != null && Core.Me.CurrentTarget.IsBehind)) return true;
            return await Spell.PullCast("True Thrust", Core.Me.CurrentTarget);
        }

        [Behavior(BehaviorType.Combat, ClassJobType.Lancer)]
        public Composite LancerCreateBasicCombat()
        {
            return new ActionRunCoroutine(ctx => LancerBuildBasicCombat());
        }
        public static async Task<bool> LancerBuildBasicCombat()
        {
            await Target.EnsureValidTarget();
            await Movement.MoveToTarget();

            Gcd = await Buddy.Coroutines.Coroutine.ExternalTask(Task.Run(() => Spell.LancerGcd()));
            await Spell.Cast("Mercy Stroke", Core.Me.CurrentTarget, Core.Me.CurrentTarget != null && Core.Me.CurrentTarget.CurrentHealthPercent < 20, false, false, true);
            if (!Gcd && await Spell.CastLock("Disembowel", Core.Me.CurrentTarget, Spell.LastCombo == Combo.ImpulseDrive)) return true;
            if (!Gcd && await Spell.CastLock("Chaos Thrust", Core.Me.CurrentTarget, Spell.LastCombo == Combo.Disembowel)) return true;
            if (!Gcd && await Spell.CastLock("Vorpal Thrust", Core.Me.CurrentTarget, Spell.LastCombo == Combo.TrueThrust)) return true;
            if (!Gcd && await Spell.CastLock("Full Thrust", Core.Me.CurrentTarget, Spell.LastCombo == Combo.VorpalThrust)) return true;
            if (!Gcd && await Spell.CastLock("Heavy Thrust", Core.Me.CurrentTarget, Core.Me.CurrentTarget != null
                && Core.Me.CurrentTarget.IsFlanking && !Core.Me.HasAura("Heavy Thrust",false,1000))) return true;
            if (!Gcd && await Spell.CastLock("Impulse Drive", Core.Me.CurrentTarget, Core.Me.CurrentTarget != null
                 && Core.Me.CurrentTarget.IsBehind && !Core.Me.CurrentTarget.HasAura("Disembowel", false, 3500))) return true;
            if (!Gcd && await Spell.CastLock("Phlebotomize", Core.Me.CurrentTarget, !Core.Me.CurrentTarget.HasAura("Phlebotomize", true, 1000))) return true;
            if (!Gcd && await Spell.Cast("Piercing Talon", Core.Me.CurrentTarget, Core.Me.CurrentTarget != null && Core.Me.CurrentTarget.Distance() > 3)) return true;
            if (!Gcd) await Spell.Cast("Haymaker", Core.Me.CurrentTarget, true, false, false, true);
            if (!Gcd && await Spell.CastLock("Virus", Core.Me.CurrentTarget, Core.Me.CurrentTarget != null && !Core.Me.CurrentTarget.HasAura("Virus"))) return true; //
            if (!Gcd && await Spell.CastLock("Venomous Bite", Core.Me.CurrentTarget, !Core.Me.CurrentTarget.HasAura("Venomous Bite", true))) return true;
            if (!Gcd && await Spell.CastLock("Fracture", Core.Me.CurrentTarget, Core.Me.CurrentTarget != null && !Core.Me.CurrentTarget.HasAura("Fracture"))) return true;
            if (!Gcd && await Spell.Cast("True Thrust", Core.Me.CurrentTarget, !Spell.MidCombo() || Core.Me.ClassLevel == 1)) return true;

            return false;
        }
    }
}