﻿//////////////////////////////////////////////////
//               Pugilist.cs                    //
//         Part of RebornCo by kaihaider        //
//////////////////////////////////////////////////
using System.Threading.Tasks;
using System.Windows.Media;
using ff14bot; 
using ff14bot.Enums;
using ff14bot.Helpers;
using ff14bot.Objects; 
using RebornCo.Helpers;
using RebornCo.Settings;
using TreeSharp;

namespace RebornCo.Rotations
{
    public partial class Routines
    {
        [Behavior(BehaviorType.PreCombatBuffs, ClassJobType.Pugilist)]
        public Composite PugilistCreateBasicPreCombatBuffs()
        {
            return new ActionRunCoroutine(ctx => RogueBuildPreCombatBuffs());
        }
        public static async Task<bool> PugilistBuildPreCombatBuffs()
        {
            if (Core.Me.IsMounted) return false;
            if (await Spell.Cast("Protect", Core.Me, !Core.Me.HasAura("Protect"), false, false, true)) return false;
            if (await Spell.Cast("Foresight", Core.Me, !Core.Me.HasAura("Foresight"), false, false, true)) return false;
            return await Movement.CombatCompanion();
        }

        [Behavior(BehaviorType.Rest, ClassJobType.Pugilist)]
        public Composite PugilistCreateBasicRest()
        {
            return new ActionRunCoroutine(ctx => DefaultRestBehavior(Core.Player.CurrentTPPercent)); 
        }


        [Behavior(BehaviorType.CombatBuffs, ClassJobType.Pugilist)]
        public Composite PugilistCreateBasicCombatBuffs()
        {
            return new ActionRunCoroutine(ctx => PugilistBuildCombatBuffs()); 
        }
        public static async Task<bool> PugilistBuildCombatBuffs()
        {
            await Spell.Cast("Perfect Dodge", Core.Me, PugSettings.CooldownsUsable && Core.Me.CurrentHealthPercent < 40, false, false, true);
            await Spell.Cast("Invigorate", Core.Me, Core.Me.CurrentTPPercent < 50, false, false, true);
            await Spell.Cast("Second Wind", Core.Me, Core.Me.CurrentHealthPercent < 40, false, false, true);
            await Spell.Cast("Awareness", Core.Me, Core.Me.CurrentHealthPercent < 30, false, false, true);
            await Spell.Cast("Bloodbath", Core.Me, Core.Me.CurrentHealthPercent < 70, false, false, true);
            await Spell.Cast("Internal Release", Core.Me, true, false, false, true);
            await Spell.Cast("Raging Strikes", Core.Me, true, false, false, true);
            await Spell.Cast("Blood For Blood", Core.Me, true, false, false, true);
            await Spell.Cast("Featherfoot", Core.Me, Core.Me.Distance2D() < 5, false, false, true);
            await Spell.Cast("Keen Flurry", Core.Me, Core.Me.CurrentHealthPercent < 80, false, false, true);
            await Spell.Cast("Foresight", Core.Me, Core.Me.CurrentHealthPercent < 60, false, false, true);
            await Spell.Cast("Convalescence", Core.Me, Core.Me.CurrentHealthPercent < 70, false, false, true);
            await Spell.Cast("Cure", Core.Me, Core.Me.CurrentHealthPercent < 40, false, false, true);
            await Spell.Cast("Physick", Core.Me, Core.Me.CurrentHealthPercent < 40, false, false, true);
            await Spell.Cast("Protect", Core.Me, !Core.Me.HasAura("Protect"), false, false, true);
            await Spell.Cast("Foresight", Core.Me, !Core.Me.HasAura("Foresight"), false, false, true);
            return false;
        }


        [Behavior(BehaviorType.Pull, ClassJobType.Pugilist)]
        public Composite PugilistCreateBasicPull()
        {
            return new ActionRunCoroutine(ctx => PugilistBuildPullBehavior()); 
        }
        public static async Task<bool> PugilistBuildPullBehavior()
        {
            await Target.EnsureValidTarget();
            if (Core.Me.HasAura("Raptor Form"))
            {
                if (await Spell.PullCast("Twin Snakes", Core.Me.CurrentTarget, Core.Me.CurrentTarget != null && Core.Me.CurrentTarget.IsBehind
                    || !Core.Me.HasAura("Twin Snakes"))) return true;
                if (await Spell.PullCast("True Strike", Core.Me.CurrentTarget)) return true;
            }
            if (Core.Me.HasAura("Coeurl Form"))
            {
                if (await Spell.PullCast("Demolish", Core.Me.CurrentTarget, Core.Me.CurrentTarget != null && 
                            !Core.Me.CurrentTarget.HasAura("Demolish",true))) return true; //
                if (await Spell.PullCast("Snap Punch", Core.Me.CurrentTarget, Core.Me.CurrentTarget != null && Core.Me.CurrentTarget.IsFlanking))
                    return true;
                if (await Spell.PullCast("Demolish", Core.Me.CurrentTarget)) return true;
                if (await Spell.PullCast("Snap Punch", Core.Me.CurrentTarget)) return true;
            }
            return await Spell.PullCast("Bootshine", Core.Me.CurrentTarget);
           // return false;
        }

        [Behavior(BehaviorType.Combat, ClassJobType.Pugilist)]
        public Composite PugilistCreateBasicCombat()
        {
            return new ActionRunCoroutine(ctx => PugilistBuildBasicCombat()); 
        }
        public static async Task<bool> PugilistBuildBasicCombat()
        {
            await Target.EnsureValidTarget();
            await Movement.MoveToTarget();
            if (PugSettings == null)
            {
                Logging.Write(Colors.OrangeRed, "Error in Settings");
                return false;
            }
            Gcd = await Buddy.Coroutines.Coroutine.ExternalTask(Task.Run(() => Spell.PugGcd()));
            await Spell.Cast("Mercy Stroke", Core.Me.CurrentTarget, Core.Me.CurrentTarget != null && Core.Me.CurrentTarget.CurrentHealthPercent < 20, false, false, true);
            if (!Gcd && await Spell.CastLock("Touch of Death", Core.Me.CurrentTarget, Core.Me.CurrentTarget != null
                && Core.Me.CurrentTarget.CurrentHealthPercent > PugSettings.MinDotHp
                       && !Core.Me.CurrentTarget.HasAura("Touch of Death",true,PugSettings.TouchOfDeathClip))) return true;
            if (!Gcd && await Spell.Cast("Perfect Balance", Core.Me, !Core.Me.HasAura("Perfect Balance")))
                return true;
            if (!Gcd && Core.Me.HasAura("Perfect Balance"))
            {
                if (await Spell.CastLock("Demolish", Core.Me.CurrentTarget, Core.Me.CurrentTarget != null
                       && !Core.Me.CurrentTarget.HasAura("Demolish", true, PugSettings.DemolishClip))) return true;
                if (await Spell.CastLock("Twin Snakes", Core.Me.CurrentTarget, !Core.Me.HasAura("Twin Snakes", false, PugSettings.TwinSnakesClip))) return true;
                if (await Spell.CastLock("Snap Punch", Core.Me.CurrentTarget)) return true;
            }
            if (!Gcd && await Spell.CastLock("Virus", Core.Me.CurrentTarget, Core.Me.CurrentTarget != null && (Core.Me.CurrentTarget as Character) != null &&
                        !(Core.Me.CurrentTarget as Character).HasAura("Virus"))) return true; 
            if (!Gcd && await Spell.Cast("Steel Peak", Core.Me.CurrentTarget)) return true;
            if (!Gcd && await Spell.Cast("Howling Fist", Core.Me.CurrentTarget, Core.Me.CurrentTarget != null && Core.Me.CurrentTarget.Distance() > Movement.MeleeRange + 3 && Core.Me.CurrentTarget.Distance() < 10f)) return true; //range
            if (!Gcd && await Spell.Cast("Venomous Bite", Core.Me.CurrentTarget, Core.Me.CurrentTarget != null && Core.Me.CurrentTarget.CurrentHealthPercent > PugSettings.MinDotHp
                        && !Core.Me.CurrentTarget.HasAura("Venomous Bite",true))) return true;
            if (!Gcd && await Spell.CastLock("Fracture", Core.Me.CurrentTarget, Core.Me.CurrentTarget != null && (Core.Me.CurrentTarget as Character) != null
                        && !(Core.Me.CurrentTarget as Character).HasAura("Fracture"))) return true;
            if (!Gcd && Core.Me.HasAura("Raptor Form"))
            {
                if (await Spell.CastLock("Twin Snakes", Core.Me.CurrentTarget, //Core.Me.CurrentTarget != null && Core.Me.CurrentTarget.IsBehind || 
                    !Core.Me.HasAura("Twin Snakes", false, PugSettings.TwinSnakesClip))) return true;
                if (await Spell.CastLock("True Strike", Core.Me.CurrentTarget)) return true;
            }
            if (!Gcd && Core.Me.HasAura("Coeurl Form"))
            {
                if (Core.Me.CurrentTarget != null && await Spell.Cast("Rockbreaker", Core.Me.CurrentTarget,
                            Core.Me.CurrentTarget.Distance() < 5 && PugSettings.Aoe && Target.EnemiesNearTarget((float)2.5) >= PugSettings.AoeCount)) return true;
                if (await Spell.CastLock("Snap Punch", Core.Me.CurrentTarget, Core.Me.CurrentTarget != null && Core.Me.CurrentTarget.IsFlanking)) return true;
                if (await Spell.CastLock("Demolish", Core.Me.CurrentTarget, !Core.Me.CurrentTarget.HasAura("Demolish", true, PugSettings.DemolishClip))) return true; //
                //if (await Spell.CastLock("Demolish", Core.Me.CurrentTarget)) return true;
                if (Core.Me.CurrentTarget != null && await Spell.Cast("Rockbreaker", Core.Me.CurrentTarget, Core.Me.CurrentTarget.Distance() < 5)) return true;
                if (await Spell.CastLock("Snap Punch", Core.Me.CurrentTarget)) return true;
            }
            if (!Gcd) await Spell.Cast("Haymaker", Core.Me.CurrentTarget, PugSettings.UseHaymaker&&Core.Me.CurrentTarget !=null && (!Core.Me.CurrentTarget.IsBehind || !Core.Me.CurrentTarget.IsFlanking),false,false,true);
            if (!Gcd && await Spell.CastLock("Bootshine", Core.Me.CurrentTarget, !(Core.Me.HasAura("Raptor Form") ||Core.Me.HasAura("Coeurl Form") ||Core.Me.HasAura("Perfect Balance")))) return true;
            return false;
        }

    }
}
