﻿
using System.Linq;
using System.Threading.Tasks;
using System.Windows.Media;
using ff14bot;
using ff14bot.Enums;
using ff14bot.Helpers;
using ff14bot.Managers;
using RebornCo.Helpers;
using RebornCo.Settings;
using TreeSharp;

namespace RebornCo.Rotations
{
    public partial class Routines
    {


        [Behavior(BehaviorType.PreCombatBuffs, ClassJobType.Rogue)]
        public Composite RogueCreateBasicPreCombatBuffs()
        {
            return new ActionRunCoroutine(ctx => RogueBuildPreCombatBuffs());
        }
        public static async Task<bool> RogueBuildPreCombatBuffs()
        {
            if (Core.Me.IsMounted) return false;
            if (await Spell.Cast("Protect", Core.Me, !Core.Me.HasAura("Protect"))) return true;
            if (await Spell.Cast("Foresight", Core.Me, !Core.Me.HasAura("Foresight"))) return true;
            if (await Spell.Cast("Kiss of the Viper", Core.Me, !(Core.Me.HasAura("Kiss of the Viper") || Core.Me.HasAura("Kiss of the Wasp")))) return true;
            if (await Spell.Cast("Kiss of the Wasp", Core.Me, !(Core.Me.HasAura("Kiss of the Viper") || Core.Me.HasAura("Kiss of the Wasp")))) return true;
            return await Movement.CombatCompanion();
        }

        [Behavior(BehaviorType.Rest, ClassJobType.Rogue)]
        public Composite RogueCreateBasicRest()
        {
            return new PrioritySelector( 
                new Decorator(ctx =>  Core.Me.HasAura("Hidden"), 
                    new ActionRunCoroutine(ctx => Spell.CastLock("Hide"))),
                new ActionRunCoroutine(ctx => DefaultRestBehavior(Core.Player.CurrentTPPercent))); 
        }


        [Behavior(BehaviorType.CombatBuffs, ClassJobType.Rogue)]
        public Composite RogueCreateBasicCombatBuffs()
        {
            return new ActionRunCoroutine(ctx => RogueBuildCombatBuffs()); 
        }
        public static async Task<bool> RogueBuildCombatBuffs()
        {
            if (await Spell.Cast("Perfect Dodge", Core.Me, RogueSettings.CooldownsUsable && Core.Me.CurrentHealthPercent < 40)) return true;
            if (await Spell.Cast("Invigorate", Core.Me, Core.Me.CurrentTPPercent < 50)) return true;
            if (await Spell.Cast("Second Wind", Core.Me, Core.Me.CurrentHealthPercent < 40)) return true;
            if (await Spell.Cast("Awareness", Core.Me, Core.Me.CurrentHealthPercent < 30)) return true;
            if (await Spell.Cast("Bloodbath", Core.Me, Core.Me.CurrentHealthPercent < 70)) return true;
            if (await Spell.Cast("Internal Release", Core.Me)) return true;
            if (await Spell.Cast("Raging Strikes", Core.Me)) return true;
            if (await Spell.Cast("Blood For Blood", Core.Me)) return true;
            if (await Spell.Cast("Featherfoot", Core.Me, Core.Me.Distance2D() < 5)) return true;
            if (await Spell.Cast("Keen Flurry", Core.Me, Core.Me.CurrentHealthPercent < 80)) return true;
            if (await Spell.Cast("Foresight", Core.Me, Core.Me.CurrentHealthPercent < 60)) return true;
            if (await Spell.Cast("Convalescence", Core.Me, Core.Me.CurrentHealthPercent < 70)) return true;
            if (await Spell.Cast("Cure", Core.Me, Core.Me.CurrentHealthPercent < 40)) return true;
            if (await Spell.Cast("Physick", Core.Me, Core.Me.CurrentHealthPercent < 40)) return true;
            if (await Spell.Cast("Protect", Core.Me, !Core.Me.HasAura("Protect"))) return true;
            return await Spell.Cast("Foresight", Core.Me, !Core.Me.HasAura("Foresight"));
        }


        [Behavior(BehaviorType.Pull, ClassJobType.Rogue)]
        public Composite RogueCreateBasicPull()
        {
            return new ActionRunCoroutine(ctx => RogueBuildPullBehavior()); 
        }
        public static async Task<bool> RogueBuildPullBehavior()
        {
            await Target.EnsureValidTarget();
            if (await Spell.Cast("Hide", Core.Me, RogueSettings.UseHide && (Core.Me.ClassLevel > 21 && !Core.Me.CurrentTarget.IsBehind && !Core.Me.CurrentTarget.IsFlanking || Core.Me.ClassLevel == 50)
                &&!Core.Me.HasAura("Hidden"))) return true;
            if (await Spell.PullCast("Sneak Attack", Core.Me.CurrentTarget, (RogueSettings.CooldownsUsable || RogueSettings.UseHide) && Core.Me.HasAura("Hidden") && Core.Me.CurrentTarget != null && !Core.Me.CurrentTarget.IsFlanking && !Core.Me.CurrentTarget.IsBehind)) return true;
            if (await Spell.PullCast("Trick Attack", Core.Me.CurrentTarget, (RogueSettings.CooldownsUsable || RogueSettings.UseHide) && Core.Me.HasAura("Hidden") && Core.Me.CurrentTarget != null && Core.Me.CurrentTarget.IsBehind)) return true;
            if (await Spell.PullCast("Shadow Fang", Core.Me.CurrentTarget, Spell.LastCombo == Combo.SpinningEdge && !Core.Me.CurrentTarget.HasAura("Shadow Fang",true))) return true;
            if (await Spell.PullCast("Gust Slash", Core.Me.CurrentTarget, Spell.LastCombo == Combo.SpinningEdge)) return true;
            if (await Spell.PullCast("Dancing Edge", Core.Me.CurrentTarget, Spell.LastCombo == Combo.GustSlash && Core.Me.CurrentTarget != null && (Core.Me.CurrentTarget.IsFlanking || !(Core.Me.CurrentTarget.HasAura("Dancing Edge") || Core.Me.CurrentTarget.HasAura("Storm's Eye"))))) return true;
            if (await Spell.PullCast("Aeolian Edge", Core.Me.CurrentTarget, Spell.LastCombo == Combo.GustSlash)) return true;
            if (await Spell.PullCast("Spinning Edge", Core.Me.CurrentTarget)) return true;
            return false;
        }

        [Behavior(BehaviorType.Combat, ClassJobType.Rogue)]
        public Composite RogueCreateBasicCombat()
        {
            return new ActionRunCoroutine(ctx => RogueBuildBasicCombat()); 
        }
        public static async Task<bool> RogueBuildBasicCombat()
        {

            await Target.EnsureValidTarget();
            await Movement.MoveToTarget();
            if (RogueSettings == null)
            {
                Logging.Write(Colors.OrangeRed, "Error in Settings");
                return false;
            }
            Gcd = await Buddy.Coroutines.Coroutine.ExternalTask(Task.Run(() => Spell.RogueGcd()));
            if (await Spell.Cast("Mercy Stroke", Core.Me.CurrentTarget, Core.Me.CurrentTarget != null && Core.Me.CurrentTarget.CurrentHealthPercent < 20)) return false;
            if (await Spell.Cast("Assassinate", Core.Me.CurrentTarget, Core.Me.CurrentTarget != null && Core.Me.CurrentTarget.CurrentHealthPercent < 20)) return false;
            if (await Spell.Cast("Sneak Attack", Core.Me.CurrentTarget, RogueSettings.CooldownsUsable && Core.Me.HasAura("Hidden") && Core.Me.CurrentTarget != null && !Core.Me.CurrentTarget.IsFlanking && !Core.Me.CurrentTarget.IsBehind)) return true;
            if (await Spell.Cast("Trick Attack", Core.Me.CurrentTarget, RogueSettings.CooldownsUsable && Core.Me.HasAura("Hidden") && Core.Me.CurrentTarget != null && Core.Me.CurrentTarget.IsBehind)) return true;
            if (!Gcd && await Spell.CastLock("Shadow Fang", Core.Me.CurrentTarget,
                        Spell.LastCombo == Combo.SpinningEdge && !Core.Me.CurrentTarget.HasAura("Shadow Fang", true,RogueSettings.ShadowFangClip) && Core.Me.CurrentTarget !=null
                        && Core.Me.CurrentTarget.CurrentHealthPercent > RogueSettings.MinDotHp)) return true;
            if (!Gcd && await Spell.Cast("Gust Slash", Core.Me.CurrentTarget, Spell.LastCombo == Combo.SpinningEdge)) return true;
            if (!Gcd && await Spell.CastLock("Dancing Edge", Core.Me.CurrentTarget,
                        Spell.LastCombo == Combo.GustSlash && Core.Me.CurrentTarget != null &&
                        (Core.Me.CurrentTarget.IsFlanking ||
                         !(Core.Me.CurrentTarget.HasAura("Dancing Edge", false, RogueSettings.DancingEdgeClip) || Core.Me.CurrentTarget.HasAura("Storm's Eye", false, RogueSettings.DancingEdgeClip))))) return true;
            if (!Gcd && await Spell.Cast("Aeolian Edge", Core.Me.CurrentTarget, Spell.LastCombo == Combo.GustSlash)) return true;
            if (!Gcd && await Spell.Cast("Death Blossom", Core.Me.CurrentTarget, RogueSettings.Aoe && Target.mNearbyEnemyUnits != null && Target.mNearbyEnemyUnits.Count(unit => unit.Distance() <= 5) >= RogueSettings.AoeCount)) return true;

            if (!Gcd && await Spell.Cast("Throwing Dagger", Core.Me.CurrentTarget, Core.Me.CurrentTarget != null && Core.Me.CurrentTarget.Distance() > Movement.MeleeRange + 4)) return true;
            if (!Gcd && await Spell.CastLock("Mutilate", Core.Me.CurrentTarget, Core.Me.CurrentTarget != null && Core.Me.CurrentTarget.CurrentHealthPercent > RogueSettings.MinDotHp
                && !Core.Me.CurrentTarget.HasAura("Mutilation", true, RogueSettings.MutilationClip))) return true;
            //if (!Gcd && await Spell.Cast("Haymaker", Core.Me.CurrentTarget)) return true;
            if (!Gcd && await Spell.Cast("Virus", Core.Me.CurrentTarget, Core.Me.CurrentTarget != null && !Core.Me.CurrentTarget.HasAura("Virus"))) return true; //
            if (!Gcd && await Spell.Cast("Impulse Drive", Core.Me.CurrentTarget, Core.Me.CurrentTarget != null && Core.Me.CurrentTarget.IsBehind)) return true;
            if (!Gcd && await Spell.CastLock("Venomous Bite", Core.Me.CurrentTarget, Core.Me.CurrentTarget != null && Core.Me.CurrentTarget.CurrentHealthPercent > RogueSettings.MinDotHp && !Core.Me.CurrentTarget.HasAura("Venomous Bite"))) return true;
            if (!Gcd && await Spell.CastLock("Fracture", Core.Me.CurrentTarget, Core.Me.CurrentTarget != null && !Core.Me.CurrentTarget.HasAura("Fracture", true))) return true;
            if (!Gcd && await Spell.CastLock("Spinning Edge", Core.Me.CurrentTarget)) return true;
            
            return false;
        }

    }
}
