﻿//////////////////////////////////////////////////
//              ArcanistSettings.cs             //
//         Part of RebornCo by kaihaider        //
//////////////////////////////////////////////////
using System.ComponentModel;
using System.Configuration;
using System.IO;
using ff14bot.Helpers;
using Newtonsoft.Json;

namespace RebornCo.Settings
{
    public class DragoonSetting : JsonSettings, ICore
    {
        [JsonIgnore]
        public static DragoonSetting _instance;

        public static DragoonSetting Instance
        {
            get { return _instance ?? (_instance = new DragoonSetting()); }
        }

        public DragoonSetting(string filename = "RebornCoSettingsDragoon")
            : base(Path.Combine(CharacterSettingsDirectory, filename + ".json"))
        {
        }

        [Setting]
        [DefaultValue(6000)]
        [Category("Dragoon")]
        public int DisembowelClip { get; set; }

        [Setting]
        [DefaultValue(6000)]
        [Category("Dragoon")]
        public int PhlebotomizeClip { get; set; }

        [Setting]
        [DefaultValue(6000)]
        [Category("Dragoon")]
        public int HeavyThrustClip { get; set; }

        #region general
        [Setting]
        [DefaultValue(true)]
        [Category("General")]
        public bool Debug { get; set; }

        [Setting]
        [DefaultValue(true)]
        [Category("General")]
        public bool MeshFreeMovement { get; set; }

        [Setting]
        [DefaultValue(true)]
        [Category("General")]
        public bool Combat { get; set; }

        [Setting]
        [DefaultValue(false)]
        [Category("General")]
        public bool Targeting { get; set; }

        [Setting]
        [DefaultValue(true)]
        [Category("General")]
        public bool Movement { get; set; }

        [Setting]
        [DefaultValue(false)]
        [Category("General")]
        public bool DontMoveIfMovinToYou { get; set; }

        [Setting]
        [DefaultValue(40f)]
        [Category("General")]
        public float RestHealth { get; set; }

        [Setting]
        [DefaultValue(40f)]
        [Category("General")]
        public float RestEnergy { get; set; }

        [Setting]
        [DefaultValue(false)]
        [Category("General")]
        public bool CombatChocobo { get; set; }

        [Setting]
        [Category("General")]
        [DefaultValue(CooldownUse.Always)]
        public CooldownUse Cooldowns { get; set; }

        [Setting]
        [Category("General")]
        [DefaultValue(false)]
        public bool Aoe { get; set; }

        [Setting]
        [DefaultValue(400)]
        [Category("Core")]
        public int GcdSyncWait { get; set; }

        [Setting]
        [DefaultValue(800)]
        [Category("Core")]
        public int SpellQueueWindow { get; set; }

        [Setting]
        [DefaultValue(3f)]
        [Category("General")]
        public float AoeCount { get; set; }

        [Setting]
        [DefaultValue(true)]
        [Category("General")]
        public bool Dodge { get; set; }

        [Setting]
        [DefaultValue(15f)]
        [Category("General")]
        public float MinDotHp { get; set; }

        [Browsable(false)]
        public virtual float RestEnergyDone
        {
            get
            {
                if (RestEnergy * 2 < 100)
                    return RestEnergy * 2;
                return 100f;
            }
        }

        [Browsable(false)]
        public virtual float RestHealthDone
        {
            get
            {
                if (RestHealth * 2 < 100)
                    return RestHealth * 2;
                return 100f;
            }
        }

        [Browsable(false)]
        public virtual bool CooldownsUsable
        {

            get
            {
                if (RebornCoRoutine.WindowSettings as DragoonSetting == null) return false;
                switch ((RebornCoRoutine.WindowSettings as DragoonSetting).Cooldowns)
                {
                    case CooldownUse.Always:
                        return true;
                    /* 
                case CooldownUse.BySpecial:
                    return Target.IsCurTargetSpecial();
case CooldownUse.ByFocus:
                   return Helpers.RebornCoHelp.me.FocusedUnit != null && Helpers.RebornCoHelp.me.FocusedUnit.Guid == Helpers.RebornCoHelp.me.CurrentTarget.Guid &&
                          !Helpers.RebornCoHelp.me.FocusedUnit.IsFriendly;
                   */
                }
                return false;
            }
        }
        #endregion
    }

}